/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.api.configuration;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import com.ebmwebsourcing.easycommons.properties.PropertiesException;
import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;

/**
 * Configuration parameters of a service unit.
 * <p>
 * An instance of this object is build with from a list of DOM elements extracted from the SU JBI descriptor. Each DOM
 * element is a configuration parameter expressed as: <code>&lt;param-name>value&lt;/param-name><code>.
 * </p>
 * <p>
 * The parameters are stored in a key/value form.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 */
public class SuConfigurationParameters extends AbstractConfigurationParameters {

    /**
     * The configuration parameters in their final form (with resolution against place holder).
     */
    private final Map<String, String> resolvedCfgParams = new ConcurrentHashMap<String, String>();

    /**
     * Place holder values
     */
    private final Properties placeHolders;

    /**
     * @param configurationElements
     *            list of DOM elements extracted from the SU JBI descriptor
     * @param placeHolders
     *            The place holder values.
     */
    public SuConfigurationParameters(final List<?> configurationElements, final Properties placeHolders) {
        super(configurationElements);

        this.placeHolders = placeHolders;
        this.reloadPlaceholderValues();
    }

    /**
     * Reload place holder values provided when creating the current instance of {@link SuConfigurationParameters}
     */
    public void reloadPlaceholderValues() {

        synchronized (this.resolvedCfgParams) {
            this.resolvedCfgParams.putAll(this.rawCfgParams);
            if (this.placeHolders != null) {
                PropertiesHelper.resolveMapWithNoException(this.resolvedCfgParams, this.placeHolders);
            }
        }
    }

    /**
     * Get the resolved value of a configuration parameter
     * 
     * @param paramName
     *            The configuration parameter name
     * @return The resolved value
     */
    @Override
    public String get(final String paramName) {
        synchronized (this.resolvedCfgParams) {
            return this.resolvedCfgParams.get(paramName);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Implementation specific: the return value is the resolved value against place holders.
     * </p>
     */
    @Override
    public String get(final String paramName, final String defaultValue) {
        final String result = this.get(paramName);
        if (result == null) {
            return defaultValue;
        } else {
            return result;
        }
    }

    @Override
    public Set<Entry<String, String>> entrySet() {
        return this.resolvedCfgParams.entrySet();
    }

    @Override
    public Set<String> keySet() {
        return this.resolvedCfgParams.keySet();
    }

    @Override
    public String put(final String key, final String value) {
        synchronized (this.resolvedCfgParams) {
            this.rawCfgParams.put(key, value);
            try {
                if (this.placeHolders != null) {
                    return this.resolvedCfgParams.put(key, PropertiesHelper.resolveString(value, this.placeHolders));
                } else {
                    return this.resolvedCfgParams.put(key, value);
                }
            } catch (final PropertiesException e) {
                return this.resolvedCfgParams.put(key, value);
            }
        }
    }

    @Override
    public String toString() {
        return this.resolvedCfgParams.toString();
    }

}
