/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.api.configuration;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;
import java.util.Properties;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * Unit tests of {@link SuConfigurationParameters}.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class SuConfigurationParametersTest {

    private static final String PLACEHOLDER_1 = "placeholder-1";

    private static final String PLACEHOLDER_1_VALUE_1 = "placeholder-1-value-1";

    private static final String PLACEHOLDER_1_VALUE_2 = "placeholder-1-value-2";

    private static final String PLACEHOLDER_2_VALUE_1 = "placeholder-2-value-1";

    private static final String PLACEHOLDER_2 = "placeholder-2";

    private static final String ELEMENT_NAME_1 = "element-1";

    private static final String ELEMENT_VALUE_1 = "element-value-1";

    private static final String ELEMENT_NAME_2 = "element-2";

    private static final String ELEMENT_NAME_3 = "element-3";

    /**
     * {@link SuConfigurationParameters} with place holders
     */
    @Test
    public void withPlaceHolders() {

        final Properties placeHolders = new Properties();
        placeHolders.setProperty(PLACEHOLDER_1, PLACEHOLDER_1_VALUE_1);

        final Document doc = DocumentBuilders.newDocument();
        final Element elt_1 = doc.createElementNS("", ELEMENT_NAME_1);
        elt_1.setTextContent(ELEMENT_VALUE_1);
        final Element elt_2 = doc.createElementNS("", ELEMENT_NAME_2);
        elt_2.setTextContent("${" + PLACEHOLDER_1 + "}");

        final SuConfigurationParameters suCfgParams = new SuConfigurationParameters(Arrays.asList(new Element[] {
                elt_1, elt_2 }), placeHolders);
        // Check that simple value and place holder are correctly retrieved
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals(PLACEHOLDER_1_VALUE_1, suCfgParams.get(ELEMENT_NAME_2));

        // Check that place holder are correctly reloaded and a property added is correctly retrieved
        suCfgParams.put(ELEMENT_NAME_3, "${" + PLACEHOLDER_2 + "}");
        placeHolders.setProperty(PLACEHOLDER_1, PLACEHOLDER_1_VALUE_2);
        suCfgParams.reloadPlaceholderValues();
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals(PLACEHOLDER_1_VALUE_2, suCfgParams.get(ELEMENT_NAME_2));
        assertEquals("${" + PLACEHOLDER_2 + "}", suCfgParams.get(ELEMENT_NAME_3));

        // Check that a property added using a place holder as value is correctly resolved
        placeHolders.setProperty(PLACEHOLDER_2, PLACEHOLDER_2_VALUE_1);
        suCfgParams.reloadPlaceholderValues();

        assertEquals(PLACEHOLDER_2_VALUE_1, suCfgParams.get(ELEMENT_NAME_3));
    }

    /**
     * {@link SuConfigurationParameters} without place holders (given as <code>null</code>)
     */
    @Test
    public void withPlaceHoldersGivenAsNull() {

        final Document doc = DocumentBuilders.newDocument();
        final Element elt_1 = doc.createElementNS("", ELEMENT_NAME_1);
        elt_1.setTextContent(ELEMENT_VALUE_1);
        final Element elt_2 = doc.createElementNS("", ELEMENT_NAME_2);
        elt_2.setTextContent("${" + PLACEHOLDER_1 + "}");

        final SuConfigurationParameters suCfgParams = new SuConfigurationParameters(Arrays.asList(new Element[] {
                elt_1, elt_2 }), null);
        // Check that simple value and place holder are correctly retrieved
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals("${" + PLACEHOLDER_1 + "}", suCfgParams.get(ELEMENT_NAME_2));

        // Check that place holder are correctly reloaded and a property added is correctly retrieved
        suCfgParams.put(ELEMENT_NAME_3, "${" + PLACEHOLDER_2 + "}");
        suCfgParams.reloadPlaceholderValues();
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals("${" + PLACEHOLDER_1 + "}", suCfgParams.get(ELEMENT_NAME_2));
        assertEquals("${" + PLACEHOLDER_2 + "}", suCfgParams.get(ELEMENT_NAME_3));
    }

    /**
     * {@link SuConfigurationParameters} with empty place holders (given as an empty property set):
     * <ul>
     * <li>until place holders are not defined, their values is not retrieved when getting the parameter values,</li>
     * <li>when a place holder is defined, its value is retrieved when getting the parameter value.</li>
     * </ul>
     */
    @Test
    public void withPlaceHoldersGivenAsEmptyPropertyset() {

        final Properties placeHolders = new Properties();

        final Document doc = DocumentBuilders.newDocument();
        final Element elt_1 = doc.createElementNS("", ELEMENT_NAME_1);
        elt_1.setTextContent(ELEMENT_VALUE_1);
        final Element elt_2 = doc.createElementNS("", ELEMENT_NAME_2);
        elt_2.setTextContent("${" + PLACEHOLDER_1 + "}");

        final SuConfigurationParameters suCfgParams = new SuConfigurationParameters(Arrays.asList(new Element[] {
                elt_1, elt_2 }), placeHolders);
        // Check that simple value and place holder are correctly retrieved
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals("${" + PLACEHOLDER_1 + "}", suCfgParams.get(ELEMENT_NAME_2));

        // Check that place holder are correctly reloaded and a property added is correctly retrieved
        placeHolders.setProperty(PLACEHOLDER_1, PLACEHOLDER_1_VALUE_2);
        suCfgParams.reloadPlaceholderValues();
        assertEquals(ELEMENT_VALUE_1, suCfgParams.get(ELEMENT_NAME_1));
        assertEquals(PLACEHOLDER_1_VALUE_2, suCfgParams.get(ELEMENT_NAME_2));
    }

}
